import java.io.PrintWriter;
import java.util.*;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.rulesys.RDFSRuleReasonerFactory;
import com.hp.hpl.jena.vocabulary.ReasonerVocabulary;

public class StudentQuery {
 
  // Namespace declarations
  static final String familyUri = "http://tinman.cs.gsu.edu#";
 

  // Jena model representing the family
  private Model model;

  
  private StudentQuery() {
   
    // Create an empty Model
    model = ModelFactory.createDefaultModel();
    model.setNsPrefix( "student", "http://tinman.cs.gsu.edu#" );
   
    String fullName = "fullname";
    String College = "collegename";
    String Major = "major";
    String Course = "course";
  
   
    // Create the types of Property we need to describe relationships
    // in the model
    Property fullname = model.createProperty(familyUri,fullName);
    Property college = model.createProperty(familyUri,College);
    Property major = model.createProperty(familyUri,Major);
    Property course = model.createProperty(familyUri,Course);


   
   
    // Create resources representing the people in our model
    Resource ranjani = model.createResource(familyUri+"Ranjani");
    Resource priyanka = model.createResource(familyUri+"Priyanka");
    Resource srujana = model.createResource(familyUri+"Srujana");
    Resource asif = model.createResource(familyUri+"Asif");
    Resource harshal = model.createResource(familyUri+"Harshal");


    // Add properties to describing the relationships between them
    ranjani.addProperty(fullname,"Ranjani Sankaran");
    priyanka.addProperty(fullname,"Chebrolu Krishna Priyanka");
    srujana.addProperty(fullname,"Srujana Gorge");
    asif.addProperty(fullname,"Mohammad Asif");
    harshal.addProperty(fullname,"Harshal Jhaveri");
   
    // Statements can also be directly created ...
    Statement statement1 = model.createStatement(ranjani,college,"gsu");
    Statement statement2 = model.createStatement(priyanka,college,"gsu");
    Statement statement3 = model.createStatement(srujana,college,"gsu");
    Statement statement4 = model.createStatement(asif,college,"gsu");
    Statement statement5 = model.createStatement(harshal,college,"gsu");

    // ... then added to the model:
    model.add(statement1);
    model.add(statement2);
    model.add(statement3);
    model.add(statement4);
    model.add(statement5);

    // Arrays of Statements can also be added to a Model:
    Statement statements[] = new Statement[5];
    statements[0] = model.createStatement(ranjani,major,"csc");
    statements[1] = model.createStatement(priyanka,major,"csc");
    statements[2] = model.createStatement(srujana,major,"csc");
    statements[3] = model.createStatement(asif,major,"csc");
    statements[4] = model.createStatement(harshal,major,"csc");
    model.add(statements);
   

    // A List of Statements can also be added
    List list = new ArrayList();

    list.add(model.createStatement(ranjani,course,"DB and the Web"));
    list.add(model.createStatement(priyanka,course,"Human Computer Interaction"));
    list.add(model.createStatement(priyanka,course,"DB and the Web"));
    list.add(model.createStatement(srujana,course,"Data Mining"));
    list.add(model.createStatement(asif,course,"Network Security"));
    list.add(model.createStatement(harshal,course,"Design and Analysis of Algorithms"));
  
    model.add(list);
    model.write(new PrintWriter(System.out));
   
   
    //*******************************************
 //List students who have taken a course
    ResIterator students_course = model.listSubjectsWithProperty(course);

    // Because subjects of statements are Resources, the method returned a ResIterator
    while (students_course.hasNext()) {

      // ResIterator has a typed nextResource() method
      Resource person = students_course.nextResource();

      // Print the URI of the resource
      System.out.println("The list of students who have taken coures"+person.getURI());
    }

    // Can also find all the courses by getting the objects of all "course" statements
    // Objects of statements could be Resources or literals, so the Iterator returned
    // contains RDFNodes
    NodeIterator courses = model.listObjectsOfProperty(course);
    System.out.println("****LIST OF COURSES***");
    while (courses.hasNext()) {

        System.out.println(courses.nextNode().toString());
      }
   
   
   
    // To find all the courses taken by a student, the model itself can be queried
    NodeIterator moreStudents1 = model.listObjectsOfProperty(priyanka, course);
    System.out.println("****LIST OF COURSES TAKEN BY PRIYANKA***");
    while (moreStudents1.hasNext()) {

        System.out.println(moreStudents1.nextNode().toString());
      }
   
    // But it's more elegant to ask the Resource directly
    // This method yields an iterator over Statements
    StmtIterator moreStudents2 = priyanka.listProperties(course);
   
    System.out.println("****LIST OF COURSES TAKEN BY PRIYANKA (USING DIRECT METHOD)***");
    while (moreStudents2.hasNext()) {

        System.out.println(moreStudents2.next().toString());
      }
 
  //*******************USING SELECTORS TO QUERY**********************
 // Find the exact statement "priyanka has taken DB and the Web"
   
    StmtIterator moreStudents5 = model.listStatements(priyanka,course,"DB and the Web");
    System.out.println("***priyanka has taken DB and the Web****");
   
    while (moreStudents5.hasNext()) {

        System.out.println(moreStudents5.next().toString());
      }
   
   
    // Find all statements with ranjani as the subject and "csc" as the object
  
    StmtIterator moreStudents6 = model.listStatements(ranjani,null,"csc");
    System.out.println("***ranjani as the subject and \"csc\" as the object****");
  
    while (moreStudents6.hasNext()) {

        System.out.println(moreStudents6.next().toString());
      }
   
   // model.write(new PrintWriter(System.out));   
    // Find any statements made about asif
   
    //StmtIterator moreStudents7 =  Model.listStatements(priyanka,null,null);
   // model.listStatements(asif,null,null);
  //  model.write(new PrintWriter(System.out));
    // Find any statement with the major property
  //  model.listStatements(null,major,null);
  //  model.write(new PrintWriter(System.out));
   Reasoner reasoner = RDFSRuleReasonerFactory.theInstance().create(null);
    reasoner.setParameter(ReasonerVocabulary.PROPsetRDFSLevel, 
                          ReasonerVocabulary.RDFS_SIMPLE);
    InfModel inf = ModelFactory.createInfModel(reasoner, model);

   
   
   
   
   
   
  }
 
  /**
   * Creates a FamilyModel and dumps the content of its RDF representation
   */
  public static void main(String args[]) {

    // Create a model representing the family
    StudentQuery thestudent = new StudentQuery();

    // Dump out a String representation of the model
   
   
   
  }
}
